@@ -3,9 +3,12 @@ import sys
 import traceback
 import glob
 import code
+import contextlib
+import pkg_resources
 
 from . import pipeline
 from . import redir
+from . import utils
 import pieshell
 
 
@@ -52,6 +55,7 @@ class Environment(object):
             redirects.redirect(2, redir.STRING)
         self._redirects = redirects
         self._clear_dir_cache()
+        self.running_pipelines = utils.LineList()
     @property
     def _exports(self):
         return self._exports_value or os.environ
@@ -114,7 +118,7 @@ class Environment(object):
             redirects = self._redirects
         res = type(self)(cwd = self._cwd, exports = exports, interactive = interactive, redirects = redirects)
         if cwd is not None:
-            res.cd(cwd)
+            res._cd(cwd)
         return res
     def __getitem__(self, name):
         """env[path] is equivalent to env(path)"""
@@ -143,6 +147,8 @@ class Environment(object):
         """Sets default redirects."""
         self._redirects.register(self._redirects._coerce(other, 'stdout'))
     def __repr__(self):
+        return "[%s:%s]" % (str(id(self))[:3], self._cwd)
+    def __str__(self):
         """Prints the current prompt"""
         if self._interactive:
             return "%s:%s >>> " % (str(id(self))[:3], self._cwd)
@@ -162,7 +168,14 @@ class Environment(object):
             self._dir_cache.extend(pipeline.BuiltinRegistry.builtins.keys())
             self._dir_cache.sort()
         return self._dir_cache
-
+    @contextlib.contextmanager
+    def temporary_env(self, env):
+        self._scope["env"] = env
+        try:
+            yield
+        finally:
+            self._scope["env"] = self
+            
 env = Environment()
 
 class EnvScope(dict):
@@ -209,21 +222,54 @@ class EnvScope(dict):
             return '<%s>' % e
 
     def execute_file(self, filename):
-        with open(filename) as f:
+        if filename.startswith("resource://"):
+            path = filename.split("://")[1]
+            pkg, path = path.split("/", 1)
+            f = pkg_resources.resource_stream(pkg, path)
+        else:
+            f = open(filename)
+        with f as f:
             content = f.read()
         code.InteractiveConsole(locals=self).runsource(content, filename, "exec")
 
     def execute_expr(self, expr):
         exec(expr, self)
 
+    def _copy_resource(self, resource, dst):
+        dstdir = os.path.dirname(dst)
+        if not os.path.exists(dstdir):
+            os.makedirs(dstdir)
+        path = resource.split("://")[1]
+        pkg, path = path.split("/", 1)
+        with pkg_resources.resource_stream(pkg, path) as inf:
+            with open(dst, "wb") as outf:
+                outf.write(inf.read())
+
+        
     def execute_startup(self):
         env = self["env"]
-        self.execute_expr("from pieshell import *")
+        self["R"] = R
+        self["Redirect"] = redir.Redirect
+        self["TMP"] = redir.TMP
+        self["STRING"] = redir.STRING
+        self["PIPE"] = redir.PIPE
+        self["PipelineError"] = pipeline.running.PipelineError
+        self["PipelineInterrupted"] = pipeline.running.PipelineInterrupted
+        self["PipelineSuspended"] = pipeline.running.PipelineSuspended
+        self["pieshell"] = pieshell
+        
+        # self.execute_expr("from pieshell import *")
         self["env"] = env
-        self.execute_expr("import readline")
         conf = os.path.expanduser('~/.config/pieshell')
-        if os.path.exists(conf):
-            self.execute_file(conf)
+        conf_resource = "resource://pieshell/resources/default_config.pysh"
+        try:
+            if not os.path.exists(conf):
+                self._copy_resource(conf_resource, conf)
+        except PermissionError:
+            self.execute_file(conf_resource)
+        else:
+            if os.path.exists(conf):
+                self.execute_file(conf)
 
     def __enter__(self):
         self.ps1 = getattr(sys, "ps1", None)
@@ -231,5 +277,5 @@ class EnvScope(dict):
 
     def __exit__(self, *args, **kw):
         sys.ps1 = self.ps1
-
+    
 envScope = EnvScope(env = env(interactive=True))
