@@ -322,10 +322,13 @@ class SignalManager(IOHandler):
                 siginfos = get_sigchlds()
 
             for siginfo in siginfos:
-                if log.debug["signal"]:
-                    log.log("Signal", "signal")
-                    for key, val in siginfo_to_names(siginfo).items():
-                        log.log("    %s: %s" % (key, val), "signal")
+                class SignalFormatter(object):
+                    def __init__(self, siginfo):
+                        self.siginfo = siginfo
+                    def __str__(self):
+                        return "Signal\n%s" % ("".join("    %s: %s\n" % (key, val)
+                                                       for key, val in siginfo_to_names(siginfo).items()),)
+                log.log(SignalFormatter(siginfo), "signal")
 
                 for key, signal_handler in list(self.signal_handlers.items()):
                     if self.match_signal(siginfo, signal_handler.filter):
@@ -376,7 +379,7 @@ def get_sigchlds():
         while True:
             (pid, status) = os.waitpid(-1, os.WUNTRACED | os.WCONTINUED | os.WNOHANG)
             if pid == 0:
-                raise StopIteration()
+                return
 
             res = {
                 "ssi_signo": 0,   # Signal number
@@ -417,7 +420,7 @@ def get_sigchlds():
 
             yield res
     except OSError:
-        raise StopIteration()
+        return
 
 class SignalHandler(object):
     def __init__(self, filter):
@@ -459,7 +462,7 @@ class ProcessSignalHandler(SignalHandler):
             self.is_running = False
             self.destroy()
             return True
-
+        
 # Generate an io-manager, hopefully for the main thread
 # If we don't have one in the main thread, signal handling won't work.
 get_io_manager()
