@@ -14,7 +14,7 @@ import re
 import builtins        
 import functools
 
-from .. import copy
+from ..utils import copy
 from .. import redir
 from .. import log
 from . import running
@@ -80,11 +80,16 @@ class Pipeline(DescribableObject):
         of another pipeline."""
         from . import redirect
         from . import pipe
+        if other is True:
+            return self
         other = self._coerce(other, 'stdout')
         if isinstance(other, redir.Redirects):
             return redirect.CmdRedirect(self._env, self, other)
         else:
             return pipe.Pipe(self._env, self, other)
+    def __and__(self, other):
+        self = self | other
+        return self.run()
     def __gt__(self, file):
         """Redirects the standard out of the pipeline to a file."""
         return self | file
@@ -92,7 +97,8 @@ class Pipeline(DescribableObject):
         """Redirects the standard in of the pipeline from a file."""
         return file | self
     def __add__(self, other):
-        return Group(self._env, self, other)
+        from . import group
+        return group.Group(self._env, self, other)
 
     def _run(self, redirects, sess, indentation = ""):
         self._started = True
@@ -107,16 +113,17 @@ class Pipeline(DescribableObject):
             processes = self._run(redirects, sess)
         pipeline = running.RunningPipeline(processes, self)
         self._env.last_pipeline = pipeline
+        self._env.running_pipelines.append(pipeline)
         return pipeline
 
     def run_interactive(self):
         pipeline = self.run()
-        try:
-            pipeline.wait()
-        except KeyboardInterrupt as e:
-            raise running.PipelineInterrupted(pipeline)
+        pipeline.wait()
         return pipeline
 
+    def __pos__(self):
+        return self.run_interactive()
+    
     def __iter__(self):
         """Runs the pipeline and iterates over its standrad output lines."""
         return iter(self.run([redir.Redirect("stdout", redir.PIPE)]))
@@ -125,7 +132,7 @@ class Pipeline(DescribableObject):
         return bytes(self).decode("utf-8")
     def __bytes__(self):
         """Runs the pipeline and returns its standrad out output as a string"""
-        return "\n".join(iter(self.run([redir.Redirect("stdout", redir.PIPE)])))
+        return b"".join(self.run([redir.Redirect("stdout", redir.PIPE)]).iterbytes())
     def __invert__(self):
         """Start a pipeline in the background"""
         return self.run()
@@ -144,20 +151,14 @@ class Pipeline(DescribableObject):
                 log.log("Canceled:\n%s" % (e,), "error")
             except running.PipelineSuspended as e:
                 log.log("Suspended:\n%s" % (e,), "error")
-            except Exception as e:
-                log.log("Error:\n%s" % (e,), "error")
-                sys.last_traceback = sys.exc_info()[2]
-                import pdb
-                pdb.pm()
             return ''
         else:
             current_env = getattr(Pipeline._print_state, 'env', None)
             Pipeline._print_state.env = self._env
             try:
-                envstr = ''
                 if current_env is not self._env:
-                    envstr = repr(self._env)
-                return "%s%s" % (envstr, self._repr())
+                    return "%s in %s" % (self._repr(), repr(self._env))
+                return self._repr()
             finally:
                 Pipeline._print_state.env = current_env
 
