@@ -17,6 +17,21 @@ import functools
 from .. import iterio
 from .. import redir
 
+try:
+    import psutil
+except:
+    psutil = None
+
+class StopSignalHandler(iterio.SignalHandler):
+    def __init__(self):
+        self.current_pipeline = None
+        iterio.SignalHandler.__init__(self, {"ssi_signo": signal.SIGTSTP})
+    def handle_event(self, event):
+        if self.current_pipeline is not None:
+            self.current_pipeline.pipeline_suspended = True
+        return True
+stop_signal_handler = StopSignalHandler()
+    
 class PipelineError(Exception):
     description = "Pipeline"
     def __init__(self, pipeline):
@@ -40,36 +55,79 @@ class RunningPipeline(object):
         self.pipeline_suspended = False
         for process in processes:
             process.running_pipeline = self
+        # Just in case all the processes have already terminated...
+        # They could have been blindingly fast after all :)
+        self.handle_finish()
     def __iter__(self):
         def handle_input():
             for line in iterio.LineInputHandler(self.pipeline._redirects.stdout.pipe, usage=self):
                 yield line
             self.wait()
         return iter(handle_input())
+    def iterbytes(self):
+        def handle_input():
+            for data in iterio.InputHandler(self.pipeline._redirects.stdout.pipe, usage=self):
+                yield data
+            self.wait()
+        return iter(handle_input())
     def restart(self):
         self.pipeline_suspended = False
         for process in self.processes:
             process.restart()
+    def suspend(self):
+        self.pipeline_suspended = True
+        for process in self.processes:
+            process.suspend()
     def wait(self):
-        self.restart()
-        # FIXME: Wake up all children here
-        while not self.pipeline_suspended and functools.reduce(operator.__or__, (proc.is_running for proc in self.processes), False):
-            iterio.get_io_manager().handle_io()
-        if self.pipeline_suspended:
-            raise PipelineSuspended(self)
-        if self.failed_processes:
-            raise PipelineFailed(self)
+        try:
+            if self.pipeline._env._interactive:
+                stop_signal_handler.current_pipeline = self
+            try:
+                self.restart()
+                while not self.pipeline_suspended and self.is_running:
+                    iterio.get_io_manager().handle_io()
+            except KeyboardInterrupt as e:
+                raise PipelineInterrupted(self)
+            if self.pipeline_suspended:
+                self.suspend()
+                raise PipelineSuspended(self)
+            if self.failed_processes:
+                raise PipelineFailed(self)
+        finally:
+            stop_signal_handler.current_pipeline = None
+    def handle_finish(self):
+        if not self.is_running:
+            for proc in self.processes:
+                proc.handle_pipeline_finish()
+            for proc in self.processes:
+                proc.handle_pipeline_finish_destructive()
+            if self in self.pipeline._env.running_pipelines:
+                self.pipeline._env.running_pipelines.remove(self)
+    def remove_output_files(self):
+        for proc in self.processes:
+            proc.remove_output_files()
     @property
     def failed_processes(self):
         return [proc
                 for proc in self.processes
                 if not proc.is_running and proc.is_failed]
-    def remove_output_files(self):
-        for proc in self.pipeline.processes:
-            proc.remove_output_files()
+    @property
+    def running_processes(self):
+        return [proc
+                for proc in self.processes
+                if proc.is_running]
     def __repr__(self):
         return repr(self.pipeline)
-
+    @property
+    def is_running(self):
+        return not not self.running_processes
+    @property
+    def is_failed(self):
+        return not not self.failed_processes
+    @property
+    def exit_code(self):
+        return self.processes[-1].exit_code
+    
 class RunningItem(object):
     def __init__(self, cmd, iohandler):
         self.cmd = cmd
@@ -83,24 +141,34 @@ class RunningItem(object):
             if not isinstance(redirect.pipe, redir.STRING): continue
             with open(redirect.pipe.path) as f:
                 self.output_content[fd] = f.read()
+        self.running_pipeline.handle_finish()
     @property
     def output_files(self):
         if self.output_content is not None: return {}
         return {fd: redirect.pipe
                 for fd, redirect in self.cmd._redirects.redirects.items()
                 if isinstance(redirect.pipe, redir.TMP) and not isinstance(redirect.pipe, redir.STRING)}
-    def remove_output_files(self):
+    def handle_pipeline_finish(self):
         for fd, redirect in self.cmd._redirects.redirects.items():
             if not isinstance(redirect.pipe, redir.STRING): continue
             # Yes, reread the STRING pipe, it can have been modified
             # by another part of the pipeline since we finished.
             with open(redirect.pipe.path) as f:
                 self.output_content[fd] = f.read()
-            os.unlink(redirect.pipe.path)        
+    def handle_pipeline_finish_destructive(self):
+        for fd, redirect in self.cmd._redirects.redirects.items():
+            if not isinstance(redirect.pipe, redir.STRING): continue
+            try:
+                os.unlink(redirect.pipe.path)
+            except FileNotFoundError:
+                pass
+    def remove_output_files(self):
         for fd, name in self.output_files.items():
             os.unlink(name)
     def restart(self):
         pass
+    def suspend(self):
+        pass
     def __getattr__(self, name):
         return getattr(self.iohandler, name)
 
@@ -124,20 +192,31 @@ class RunningProcess(RunningItem):
             self.process = process
             iterio.ProcessSignalHandler.__init__(self, pid)
         def handle_event(self, event):
+            res = iterio.ProcessSignalHandler.handle_event(self, event)
             if not self.is_running:
                 self.process.handle_finish()
-            if event["ssi_signo"] == signal.SIGCHLD and event["ssi_code"] == iterio.CLD_STOPPED:
-                self.process.running_pipeline.pipeline_suspended = True
-                return True
-            else:
-                return iterio.ProcessSignalHandler.handle_event(self, event)
+            return res
     def __init__(self, cmd, pid):
         RunningItem.__init__(self, cmd, self.ProcessSignalHandler(self, pid))
     def restart(self):
-        os.kill(self.iohandler.pid, signal.SIGCONT)
+        try:
+            os.kill(self.iohandler.pid, signal.SIGCONT)
+        except ProcessLookupError:
+            pass
+    def suspend(self):
+        try:
+            os.kill(self.iohandler.pid, signal.SIGSTOP)
+        except ProcessLookupError:
+            pass
+    @property
+    def pid(self):
+        return self.iohandler.pid
     @property
     def is_failed(self):
-        return self.iohandler.last_event["ssi_status"] != 0
+        return self.exit_code != 0
+    @property
+    def exit_code(self):
+        return self.iohandler.last_event["ssi_status"]
     def __repr__(self, display_output=False):
         status = []
         last_event = self.iohandler.last_event
@@ -163,3 +242,11 @@ class RunningProcess(RunningItem):
                 fd = redir.Redirect.names_to_fd.get(fd, fd)
                 res += "%s content:\n%s\n" % (fd, value) 
         return res
+    @property
+    def details(self):
+        if psutil is None: return None
+        return psutil.Process(self.pid)
+    def __getattr__(self, name):
+        if psutil is None:
+            raise AttributeError(name)
+        return getattr(self.details, name)
