@@ -7,9 +7,9 @@ import uuid
 import code
 import threading
 
-from . import pipe
+from .utils import pipe
 from . import log
-from . import copy
+from .utils import copy
 
 try:
     MAXFD = os.sysconf("SC_OPEN_MAX")
